@@ -2,7 +2,7 @@
 use crate::utils::{
     cal3BV, cal3BV_exp, cal_table_minenum_enum, cal_table_minenum_recursion, combine, enuOneStep,
     enum_comb, lay_mine_op_number, lay_mine_number, legalize_board, refreshBoard, refresh_matrix,
-    refresh_matrixs, sum, unsolvableStructure, BigNumber, C_query, C,
+    refresh_matrixs, sum, unsolvable_structure, BigNumber, C_query, C,
 };
 
 
@@ -30,8 +30,8 @@ use tract_onnx::prelude::*;
 // 中高级的算法，例如无猜埋雷、判雷引擎、计算概率
 
 
-/// 双集合判雷引擎，除判雷外，会在局面上标是雷（11），但不标非雷（12）
-pub fn SolveMinus(
+/// 双集合判雷引擎，除判雷外，还会修改输入的局面，在局面上标是雷（11），但不标非雷（12）
+pub fn solve_minus(
     MatrixA: &Vec<Vec<i32>>,
     Matrixx: &Vec<(usize, usize)>,
     Matrixb: &Vec<i32>,
@@ -89,8 +89,7 @@ pub fn SolveMinus(
 }
 
 /// 单集合判雷引擎，除判雷外，会在局面上标是雷（11），但不标非雷（12）
-#[cfg(any(feature = "py", feature = "js", feature = "rs"))]
-pub fn SolveDirect(
+pub fn solve_direct(
     MatrixA: &mut Vec<Vec<i32>>,
     Matrixx: &mut Vec<(usize, usize)>,
     Matrixb: &mut Vec<i32>,
@@ -141,11 +140,11 @@ pub fn SolveDirect(
     (NotMine, flag)
 }
 
-/// 输入局面、未被标出的雷数
-/// 局面中可以标雷（11），但必须全部标对
-/// 未知雷数 = 总雷数 - 已经标出的雷
-/// 若超出枚举长度，返回空向量
-/// 返回所有边缘格子是雷的概率、内部未知格子是雷的概率、局面中总未知雷数的范围
+/// 输入局面、未被标出的雷数  
+/// 局面中可以标雷（11），但必须全部标对  
+/// 未知雷数 = 总雷数 - 已经标出的雷  
+/// 若超出枚举长度，返回空向量  
+/// 返回所有边缘格子是雷的概率、内部未知格子是雷的概率、局面中总未知雷数的范围  
 /// 若没有内部未知区域，返回NaN
 pub fn cal_possibility(
     board_of_game: &Vec<Vec<i32>>,
@@ -161,8 +160,6 @@ pub fn cal_possibility(
     // 例如：[[[17, 18, 19, 20, 21, 22, 23, 24], [48, 2144, 16872, 49568, 68975, 48960, 16608, 2046]]]
     let (matrix_a_s, matrix_x_s, matrix_b_s, unknow_block, is_mine_num) =
         refresh_matrixs(&board_of_game);
-    // println!("matrix_x_s: {:?}", matrix_x_s);
-    // println!("board_of_game: {:?}", board_of_game);
     let block_num = matrix_a_s.len(); // 整个局面被分成的块数
 
     let mut matrixA_squeeze_s: Vec<Vec<Vec<i32>>> = vec![];
@@ -175,7 +172,6 @@ pub fn cal_possibility(
         matrixA_squeeze_s.push(matrixA_squeeze);
         matrixx_squeeze_s.push(matrixx_squeeze);
     }
-    // println!("组合情况：{:?}", comb_relp_s);
     // 分块枚举后，根据雷数限制，删除某些情况
     for i in 0..block_num {
         let table_minenum_i;
@@ -204,15 +200,8 @@ pub fn cal_possibility(
         min_mine_num += table_minenum_s[i][0].iter().min().unwrap();
         max_mine_num += table_minenum_s[i][0].iter().max().unwrap();
     }
-    // println!("table_minenum: {:?}", table_minenum_s);
-    // println!("table_cell_minenum:{:?}", table_cell_minenum_s);
-    // println!("min_mine_num: {:?}", min_mine_num);
-    // println!("max_mine_num:{:?}", max_mine_num);
     let mine_num = if mine_num <= 1.0 {
         let mn = ((board_of_game.len() * board_of_game[0].len()) as f64 * mine_num) as usize;
-        // println!("mn:{:?}", mn);
-        // println!("unknow_block:{:?}", unknow_block);
-        // println!("is_mine_num:{:?}", is_mine_num);
         min(
             max(mn - is_mine_num, min_mine_num),
             max_mine_num + unknow_block,
@@ -222,7 +211,6 @@ pub fn cal_possibility(
     };
 
     max_mine_num = min(max_mine_num, mine_num);
-    // println!("mine_num = {:?}; min_mine_num = {:?}; max_mine_num = {:?}", mine_num, min_mine_num, max_mine_num);
     let unknow_mine_num: Vec<usize> =
         (mine_num - max_mine_num..min(mine_num - min_mine_num, unknow_block) + 1).collect();
     // 这里的写法存在极小的风险，例如边缘格雷数分布是0，1，3，而我们直接认为了可能有2
@@ -238,8 +226,6 @@ pub fn cal_possibility(
         .map(|i| 0..table_minenum_s[i][0].len())
         .multi_cartesian_product()
         .collect::<Vec<_>>();
-    // println!("mine_in_each_block: {:?}", mine_in_each_block);
-    // println!("table_minenum_s: {:?}", table_minenum_s);
     for i in (0..mine_in_each_block.len()).rev() {
         let mut total_num = 0;
         for j in 0..block_num + 1 {
@@ -281,9 +267,6 @@ pub fn cal_possibility(
         let ps = unknow_mine_num.iter().position(|x| *x == s_mn).unwrap();
         table_minenum_other[block_num][ps].add_big_number(&s_num);
     }
-    // println!("table_minenum_s: {:?}", table_minenum_s);
-    // println!("table_cell_minenum_s: {:?}", table_cell_minenum_s);
-    // println!("table_minenum_other: {:?}", table_minenum_other);
     // 第四步，计算每块其他雷数情况表
     let mut T = BigNumber { a: 0.0, b: 0 };
     for i in 0..unknow_mine_s_num.len() {
@@ -306,10 +289,6 @@ pub fn cal_possibility(
                 let p_cell = s_cell.div_big_num(&T);
                 let id = comb_relp_s[i][cells_id][cell_id];
                 p.push((matrix_x_s[i][id], p_cell));
-                // println!("p_cell: {:?}", p_cell);
-                // println!("s_cell: {:?}", s_cell);
-                // println!("T: {:?}", T);
-                // println!("table_cell_minenum: {:?}", table_cell_minenum);
             }
         }
     }
@@ -321,8 +300,6 @@ pub fn cal_possibility(
         u.mul_usize(unknow_mine_num[i]);
         u_s.add_big_number(&u);
     }
-    // println!("unknow_mine_num: {:?}", unknow_mine_num);
-    // println!("unknow_mine_s_num: {:?}", unknow_mine_s_num);
     let p_unknow = u_s.div_big_num(&T) / unknow_block as f64;
     // 第七步，计算内部未知区域是雷的概率
 
@@ -365,9 +342,10 @@ pub fn cal_possibility_onboard(
     Ok((p, pp.2))
 }
 
-/// 计算单点开空概率
-/// 输入局面、未被标出的雷数、坐标
-/// 返回坐标处开空的概率
+/// 计算单点开空概率  
+/// 输入局面、未被标出的雷数、坐标  
+/// 返回坐标处开空的概率  
+/// # Example
 /// ```
 /// let game_board = vec![
 ///     vec![10, 10,  1,  1, 10,  1,  0,  0],
@@ -419,13 +397,14 @@ pub fn cal_is_op_possibility_cells(
     poss
 }
 
-/// 埋雷，使得起手位置必为空
-/// 输入行数、列数、雷数、起手位置行数、起手位置列数、最小3BV、最大3BV、最大尝试次数、模式（保留）
-/// 输出局面和参数，分别为：是否满足3BV上下限、3BV、尝试次数
+/// 埋雷，使得起手位置必为空  
+/// 输入行数、列数、雷数、起手位置行数、起手位置列数、最小3BV、最大3BV、最大尝试次数、模式（保留）  
+/// 输出局面和参数，分别为：是否满足3BV上下限、3BV、尝试次数  
+/// # Example
 /// ```
 /// layMineOp(16, 30, 99, 0, 0, 100, 381, 10000, 0)
 /// ```
-pub fn layMineOp(
+pub fn laymine_op(
     row: usize,
     column: usize,
     MineNum: usize,
@@ -457,20 +436,17 @@ pub fn layMineOp(
     (Board, Parameters)
 }
 
-/// 枚举法判雷引擎
-/// 输入分块好的矩阵、局面、枚举长度限制
-/// 输出不是雷的位置
-/// 注意，不修改输入进来的局面，即不帮助标雷
-pub fn SolveEnumerate(
+/// 枚举法判雷引擎  
+/// 输入分块好的矩阵、局面、枚举长度限制  
+/// 输出不是雷的位置  
+/// 注意：不修改输入进来的局面，即不帮助标雷
+pub fn solve_enumerate(
     matrix_as: &Vec<Vec<Vec<i32>>>,
     matrix_xs: &Vec<Vec<(usize, usize)>>,
     matrix_bs: &Vec<Vec<i32>>,
     board_of_game: &Vec<Vec<i32>>,
     enuLimit: usize,
 ) -> (Vec<(usize, usize)>, bool) {
-    // print!("matrix_xs: {:?}", matrix_xs);
-    // print!("matrix_as: {:?}", matrix_as);
-    // print!("matrix_bs: {:?}", matrix_bs);
     let mut flag = false;
     let mut NotMine = vec![];
     let block_num = matrix_xs.len();
@@ -484,14 +460,10 @@ pub fn SolveEnumerate(
     for i in 0..block_num {
         let (matrixA_squeeze, matrixx_squeeze, combination_relationship) =
             combine(&matrix_as[i], &matrix_xs[i]);
-        // print!("**matrixA_squeeze: {:?}", matrixA_squeeze);
-        // print!("**matrixx_squeeze: {:?}", matrixx_squeeze);
         comb_relp_s.push(combination_relationship);
         matrixA_squeeze_s.push(matrixA_squeeze);
         matrixx_squeeze_s.push(matrixx_squeeze);
     }
-    // print!("77matrixx_squeeze_s: {:?}", matrixx_squeeze_s);
-    // print!("77matrixA_squeeze_s: {:?}", matrixA_squeeze_s);
     for i in 0..block_num {
         let (table_minenum_i, table_cell_minenum_i) = cal_table_minenum_recursion(
             &matrixA_squeeze_s[i],
@@ -506,8 +478,6 @@ pub fn SolveEnumerate(
                     continue 'outer;
                 }
             }
-            // println!("comb_relp_s: {:?}", comb_relp_s);
-            // println!("matrixx_squeeze_s: {:?}", matrixx_squeeze_s);
             for kk in &comb_relp_s[i][jj] {
                 NotMine.push(matrix_xs[i][*kk]);
             }
@@ -521,9 +491,9 @@ pub fn SolveEnumerate(
 // 这个函数不具备普遍意义
 fn isVictory(BoardofGame: &Vec<Vec<i32>>, Board: &Vec<Vec<i32>>) -> bool {
     let row = BoardofGame.len();
-    let Col = BoardofGame[0].len();
+    let col = BoardofGame[0].len();
     for i in 0..row {
-        for j in 0..Col {
+        for j in 0..col {
             if BoardofGame[i][j] == 10 && Board[i][j] != -1 {
                 return false;
             }
@@ -532,10 +502,10 @@ fn isVictory(BoardofGame: &Vec<Vec<i32>>, Board: &Vec<Vec<i32>>) -> bool {
     return true;
 }
 
-/// 从指定位置开始扫，判断局面是否无猜
+/// 从指定位置开始扫，判断局面是否无猜  
 /// 注意：周围一圈都是雷，那么若中间是雷不算猜，若中间不是雷算猜
-pub fn isSolvable(Board: &Vec<Vec<i32>>, x0: usize, y0: usize, enuLimit: usize) -> bool {
-    if unsolvableStructure(&Board) {
+pub fn is_solvable(Board: &Vec<Vec<i32>>, x0: usize, y0: usize, enuLimit: usize) -> bool {
+    if unsolvable_structure(&Board) {
         //若包含不可判雷结构，则不是无猜
         return false;
     }
@@ -553,18 +523,18 @@ pub fn isSolvable(Board: &Vec<Vec<i32>>, x0: usize, y0: usize, enuLimit: usize)
     let mut flag;
     loop {
         let (mut MatrixA, mut Matrixx, mut Matrixb) = refresh_matrix(&BoardofGame);
-        let ans = SolveDirect(&mut MatrixA, &mut Matrixx, &mut Matrixb, &mut BoardofGame);
+        let ans = solve_direct(&mut MatrixA, &mut Matrixx, &mut Matrixb, &mut BoardofGame);
         NotMine = ans.0;
         flag = ans.1;
         if !flag {
-            let ans = SolveMinus(&MatrixA, &Matrixx, &Matrixb, &mut BoardofGame);
+            let ans = solve_minus(&MatrixA, &Matrixx, &Matrixb, &mut BoardofGame);
             NotMine = ans.0;
             flag = ans.1;
             if !flag {
                 let (mut Matrix_as, mut Matrix_xs, mut Matrix_bs, _, _) =
                     refresh_matrixs(&BoardofGame);
                 // print!("BoardofGame: {:?}", BoardofGame);
-                let ans = SolveEnumerate(
+                let ans = solve_enumerate(
                     &Matrix_as,
                     &Matrix_xs,
                     &Matrix_bs,
@@ -585,7 +555,11 @@ pub fn isSolvable(Board: &Vec<Vec<i32>>, x0: usize, y0: usize, enuLimit: usize)
     }
 }
 
-pub fn layMineSolvable_thread(
+/// 删选法多（10）线程无猜埋雷  
+/// 3BV下限、上限，最大尝试次数，返回是否成功  
+/// 若不成功返回最后生成的局面，此时则不一定无猜
+#[cfg(any(feature = "py", feature = "rs"))]
+pub fn laymine_solvable_thread(
     row: usize,
     column: usize,
     MineNum: usize,
@@ -596,7 +570,6 @@ pub fn layMineSolvable_thread(
     mut MaxTimes: usize,
     enuLimit: usize,
 ) -> (Vec<Vec<i32>>, [usize; 3]) {
-    // 利用删选法，多线程无猜埋雷
     let mut parameters = [0, 0, 0];
     let mut game_board = vec![vec![0; column]; row];
     let mut handles = vec![];
@@ -621,7 +594,7 @@ pub fn layMineSolvable_thread(
                 } // 这块用花括号控制生命周期
                 let Board_ = lay_mine_op_number(row, column, MineNum, x0, y0);
                 counter += 1;
-                if isSolvable(&Board_, x0, y0, enuLimit) {
+                if is_solvable(&Board_, x0, y0, enuLimit) {
                     Num3BV = cal3BV(&Board_);
                     if Num3BV >= Min3BV && Num3BV <= Max3BV {
                         para[0] = 1;
@@ -663,8 +636,10 @@ pub fn layMineSolvable_thread(
     (game_board, parameters)
 }
 
-#[cfg(any(feature = "py", feature = "js", feature = "rs"))]
-pub fn layMineSolvable(
+/// 删选法单线程无猜埋雷  
+/// 3BV下限、上限，最大尝试次数，返回是否成功  
+/// 若不成功返回最后生成的局面，此时则不一定无猜
+pub fn laymine_solvable(
     row: usize,
     column: usize,
     MineNum: usize,
@@ -675,9 +650,6 @@ pub fn layMineSolvable(
     MaxTimes: usize,
     enuLimit: usize,
 ) -> (Vec<Vec<i32>>, Vec<usize>) {
-    // 单线程埋雷无猜
-    // 3BV下限、上限，最大尝试次数，返回是否成功。
-    // 若不成功返回最后生成的局面（不一定无猜），默认尝试十万次
     let mut times = 0;
     let mut Parameters = vec![];
     let mut Board;
@@ -685,7 +657,7 @@ pub fn layMineSolvable(
     while times < MaxTimes {
         Board = lay_mine_op_number(row, column, MineNum, x0, y0);
         times += 1;
-        if isSolvable(&Board, x0, y0, enuLimit) {
+        if is_solvable(&Board, x0, y0, enuLimit) {
             Num3BV = cal3BV(&Board);
             if Num3BV >= Min3BV && Num3BV <= Max3BV {
                 Parameters.push(1);
@@ -703,7 +675,7 @@ pub fn layMineSolvable(
     (Board, Parameters)
 }
 
-#[cfg(any(feature = "py", feature = "js", feature = "rs"))]
+/// 调整法无猜埋雷（没写好）
 pub fn lay_mine_solvable_adjust(
     row: usize,
     column: usize,
@@ -730,7 +702,7 @@ pub fn lay_mine_solvable_adjust(
     }
     if row * column - area_op < mine_num {
         // 雷数太多以致起手无法开空，此时放弃无猜，返回任意一种局面
-        let t = layMine(
+        let t = laymine(
             row, column, mine_num, x0, y0, min_3BV, max_3BV, max_times, 0,
         );
         return (t.0, [0, t.1[1], t.1[2]]);
@@ -741,8 +713,12 @@ pub fn lay_mine_solvable_adjust(
     (vec![], [0, 0, 0])
 }
 
-#[cfg(any(feature = "py", feature = "js", feature = "rs"))]
-pub fn layMine(
+/// 标准埋雷，参数依次是行、列、雷数、起手位置的第几行、第几列  
+/// 适用于游戏的埋雷算法  
+/// 起手不开空，必不为雷  
+/// 返回二维列表，0~8代表数字，-1代表雷  
+/// method = 0筛选算法；1调整算法（保留）
+pub fn laymine(
     row: usize,
     column: usize,
     MineNum: usize,
@@ -753,11 +729,7 @@ pub fn layMine(
     MaxTimes: usize,
     method: usize,
 ) -> (Vec<Vec<i32>>, Vec<usize>) {
-    // 埋雷，参数依次是行、列、雷数、起手位置的第几行-1、第几列-1
-    // 适用于游戏的埋雷算法。
-    // 起手不开空，必不为雷
-    // 返回二维列表，0~8代表数字，-1代表雷
-    // method = 0筛选算法；1调整算法（保留）
+    
     let mut times = 0;
     let mut Parameters = vec![];
     let mut Num3BV = 0;
@@ -883,15 +855,35 @@ fn OBR_cell(
     }
 }
 
+/// 输入列向量形式的三通道的像素数据，图像的高度、宽度；  
+/// 以下提供一段用python调用时的demo  
+/// ```python
+/// import ms_toollib
+/// import matplotlib.image as mpimg
+/// import numpy as np
+/// file = r'C:\Users\jia32\Desktop\无标题.png'# 彩色图片
+/// lena = mpimg.imread(file)
+/// (height, width) = lena.shape[:2]
+/// lena = np.concatenate((lena, 255.0 * np.ones((height, width, 1))), 2)
+/// lena = (np.reshape(lena, -1) * 255).astype(np.uint32)
+/// board = ms_toollib.py_OBR_board(lena, height, width)
+/// print(np.array(board))# 打印识别出的局面
+/// poss = ms_toollib.py_cal_possibility(board, 99)
+/// print(poss)# 用雷的总数计算概率
+/// poss_onboard = ms_toollib.py_cal_possibility_onboard(board, 99)
+/// print(poss_onboard)# 用雷的总数计算概率，输出局面对应的位置
+/// poss_ = ms_toollib.py_cal_possibility_onboard(board, 0.20625)
+/// print(poss_)# 用雷的密度计算概率
+/// ```
+/// 注意：必须配合"params.onnx"参数文件调用  
+/// 注意：由于利用了神经网络技术，可能发生识别错误，此时输出是不一定合法的局面
 #[cfg(any(feature = "py", feature = "rs"))]
 pub fn OBR_board(
     data_vec: Vec<usize>,
     height: usize,
     width: usize,
 ) -> Result<Vec<Vec<i32>>, String> {
-    // 输入列向量形式的三通道的像素数据，图像的高度、宽度；
-    // 为什么输入形式这么奇怪呢？是为了适配python截图出来的原始数据
-    // 输出是不一定合法的局面
+    // 为什么输入形式这么奇怪呢？主要是为了适配python截图出来的原始数据格式
     if height <= 24 || width <= 24 {
         return Err("one input size of the board is smaller than 3".to_string());
     }
@@ -923,44 +915,42 @@ pub fn OBR_board(
     Ok(board)
 }
 
-#[cfg(any(feature = "py", feature = "js", feature = "rs"))]
+/// 对局面用单集合、双集合判雷引擎，快速标雷、标非雷，以供概率计算引擎处理  
+/// 相当于一种预处理，即先标出容易计算的
 pub fn mark_board(board: &mut Vec<Vec<i32>>) {
-    // 对局面用单集合、双集合判雷引擎，快速标雷、标非雷
-    // 以供概率计算引擎处理
-    // 相当于一种预处理，先标出容易计算的
+    
     let (mut matrix_a, mut matrix_x, mut matrix_b) = refresh_matrix(&board);
-    let ans = SolveDirect(&mut matrix_a, &mut matrix_x, &mut matrix_b, board);
+    let ans = solve_direct(&mut matrix_a, &mut matrix_x, &mut matrix_b, board);
     let not_mine = ans.0;
     for i in not_mine {
         board[i.0][i.1] = 12;
     }
     let (matrix_a, matrix_x, matrix_b) = refresh_matrix(&board);
-    let ans = SolveMinus(&matrix_a, &matrix_x, &matrix_b, board);
+    let ans = solve_minus(&matrix_a, &matrix_x, &matrix_b, board);
     let not_mine = ans.0;
     for i in not_mine {
         board[i.0][i.1] = 12;
     }
 }
 
-#[cfg(any(feature = "py", feature = "js", feature = "rs"))]
 /// 求出所有非雷的位置
 pub fn solve_all_notmine_on_board(board_: &Vec<Vec<i32>>, enuLimit: usize) -> Vec<(usize, usize)> {
     let mut board = board_.clone();
     let (mut matrix_a, mut matrix_x, mut matrix_b) = refresh_matrix(&board);
-    let ans = SolveDirect(&mut matrix_a, &mut matrix_x, &mut matrix_b, &mut board);
+    let ans = solve_direct(&mut matrix_a, &mut matrix_x, &mut matrix_b, &mut board);
     let mut not_mine = vec![];
     for i in ans.0 {
         board[i.0][i.1] = 12;
         not_mine.push(i);
     }
     let (matrix_a, matrix_x, matrix_b) = refresh_matrix(&board);
-    let ans = SolveMinus(&matrix_a, &matrix_x, &matrix_b, &mut board);
+    let ans = solve_minus(&matrix_a, &matrix_x, &matrix_b, &mut board);
     for i in ans.0 {
         board[i.0][i.1] = 12;
         not_mine.push(i);
     }
     let (matrix_as, matrix_xs, matrix_bs, _, _) = refresh_matrixs(&board);
-    let ans = SolveEnumerate(&matrix_as, &matrix_xs, &matrix_bs, &mut board, enuLimit);
+    let ans = solve_enumerate(&matrix_as, &matrix_xs, &matrix_bs, &mut board, enuLimit);
     for i in ans.0 {
         board[i.0][i.1] = 12;
         not_mine.push(i);
