@@ -4,12 +4,14 @@ use rand::seq::SliceRandom;
 #[cfg(any(feature = "py", feature = "rs"))]
 use rand::thread_rng;
 use std::cmp::{max, min};
-use std::convert::TryInto;
+// use std::convert::TryInto;
+#[cfg(feature = "js")]
+use getrandom::getrandom;
 
 // 整个模块是最底层的一些小工具，如埋雷、局面分块、计算3BV等
 
-pub fn calOp(mut Board: Vec<Vec<i32>>) -> usize {
-    // 输入局面，计算空，即0的8连通域数
+/// 输入局面，计算空，即0的8连通域数
+pub fn cal_op(mut Board: Vec<Vec<i32>>) -> usize {
     let row = Board.len();
     let column = Board[0].len();
     let mut Op = 0;
@@ -40,12 +42,11 @@ fn infectBoard(mut Board: Vec<Vec<i32>>, x: usize, y: usize) -> Vec<Vec<i32>> {
     Board
 }
 
-/// 输入的必须保证是合法的游戏局面
+/// 输入的必须保证是合法的游戏局面  
 /// 根据游戏局面生成矩阵，不分块
 pub fn refresh_matrix(
     BoardofGame: &Vec<Vec<i32>>,
 ) -> (Vec<Vec<i32>>, Vec<(usize, usize)>, Vec<i32>) {
-    
     let row = BoardofGame.len();
     let column = BoardofGame[0].len();
     let mut MatrixA: Vec<Vec<i32>> = Vec::new();
@@ -99,7 +100,7 @@ pub fn refresh_matrix(
     (MatrixA, Matrixx, Matrixb)
 }
 
-/// 输入的必须保证是合法的游戏局面
+/// 输入的必须保证是合法的游戏局面  
 /// 根据游戏局面生成矩阵，分块
 pub fn refresh_matrixs(
     board_of_game: &Vec<Vec<i32>>,
@@ -233,7 +234,45 @@ pub fn refresh_matrixs(
     (matrix_as, matrix_xs, matrix_bs, unknow_block, is_mine_num)
 }
 
-/// 通用标准埋雷引擎，起手位置非类，其余位置的雷服从均匀分布
+// 获取0~limit-1范围内的随机整数
+// 用于js平台
+#[cfg(feature = "js")]
+fn get_random_int(limit: usize) -> usize {
+    if limit == 0 {
+        return 0;
+    }
+    let mut a = [0, 0, 0, 0];
+    let mut t;
+    loop {
+        getrandom(&mut a).unwrap();
+        println!("{:?}", a);
+        t = (a[0] as usize) << 24 ^ (a[1] as usize) << 16 ^ (a[2] as usize) << 8 ^ (a[3] as usize);
+        if t < (0b11111111_11111111_11111111_11111111 / limit * limit) {
+            break;
+        }
+    }
+    t % limit
+}
+
+#[cfg(feature = "js")]
+trait js_shuffle {
+    fn shuffle_(&mut self);
+}
+
+#[cfg(feature = "js")]
+impl js_shuffle for Vec<i32> {
+    fn shuffle_(&mut self) {
+        let l = self.len();
+        for i in 1..l {
+            let id = get_random_int(i + 1);
+            let t = self[i];
+            self[i] = self[id];
+            self[id] = t;
+        }
+    }
+}
+
+/// 通用标准埋雷引擎，起手位置非雷，其余位置的雷服从均匀分布  
 /// 输出为二维的局面，其中0代表空，1~8代表1~8，-1代表雷
 pub fn lay_mine_number(
     row: usize,
@@ -254,6 +293,9 @@ pub fn lay_mine_number(
     #[cfg(any(feature = "py", feature = "rs"))]
     Board1Dim.shuffle(&mut rng);
 
+    #[cfg(feature = "js")]
+    Board1Dim.shuffle_();
+
     let mut Board1Dim_2: Vec<i32> = vec![];
     Board1Dim_2.reserve(area + 1);
     let pointer = X0 + Y0 * row;
@@ -282,6 +324,8 @@ pub fn lay_mine_number(
     Board
 }
 
+/// 通用win7埋雷引擎，起手位置开空，其余位置的雷服从均匀分布  
+/// 输出为二维的局面，其中0代表空，1~8代表1~8，-1代表雷
 pub fn lay_mine_op_number(
     row: usize,
     column: usize,
@@ -289,7 +333,6 @@ pub fn lay_mine_op_number(
     X0: usize,
     Y0: usize,
 ) -> Vec<Vec<i32>> {
-    
     let mut areaOp = 9;
     if X0 == 0 || Y0 == 0 || X0 == row - 1 || Y0 == column - 1 {
         if X0 == 0 && Y0 == 0
@@ -308,9 +351,13 @@ pub fn lay_mine_op_number(
 
     #[cfg(any(feature = "py", feature = "rs"))]
     let mut rng = thread_rng();
+
     #[cfg(any(feature = "py", feature = "rs"))]
     Board1Dim.shuffle(&mut rng);
 
+    #[cfg(feature = "js")]
+    Board1Dim.shuffle_();
+
     let mut Board = vec![vec![0; column]; row];
     let mut skip = 0;
     for i in 0..(area + areaOp) {
@@ -361,7 +408,7 @@ fn cal3BVonIsland(Board: &Vec<Vec<i32>>) -> usize {
 
 /// 计算局面的3BV
 pub fn cal3BV(Board: &Vec<Vec<i32>>) -> usize {
-    cal3BVonIsland(&Board) + calOp(Board.clone())
+    cal3BVonIsland(&Board) + cal_op(Board.clone())
 }
 
 /// 依据左击位置刷新局面
@@ -522,8 +569,6 @@ pub fn combine(
     let mut matrixA_squeeze = MatrixA.clone();
     let mut matrixx_squeeze = Matrixx.clone();
     let cells_num = matrixx_squeeze.len();
-    // println!("matrixA_squeeze: {:?}", matrixA_squeeze);
-    // println!("matrixx_squeeze: {:?}", matrixx_squeeze);
     let mut pair_cells = vec![];
     let mut del_cells = vec![]; // 由于重复需要最后被删除的列
     for i in 0..cells_num {
@@ -535,8 +580,6 @@ pub fn combine(
             }
         }
     }
-    // println!("del_cells: {:?}", del_cells);
-    // println!("pair_cells: {:?}", pair_cells);
     del_cells.sort_by(|a, b| b.cmp(&a));
     del_cells.dedup();
     for i in del_cells {
@@ -553,12 +596,9 @@ pub fn combine(
             matrixA_squeeze[r][i] *= k;
         }
     }
-    // println!("**matrixA_squeeze: {:?}", matrixA_squeeze);
-    // println!("**matrixx_squeeze: {:?}", matrixx_squeeze);
     (matrixA_squeeze, matrixx_squeeze, pair_cells)
 }
 
-#[cfg(any(feature = "py", feature = "js", feature = "rs"))]
 pub fn enum_comb(
     matrixA_squeeze: &Vec<Vec<i32>>,
     matrixx_squeeze: &Vec<(usize, usize)>,
@@ -591,7 +631,6 @@ pub fn enum_comb(
             .multi_cartesian_product()
             .collect::<Vec<_>>();
         new_enum_table.retain(|x| x.iter().sum::<i32>() <= Matrixb[row]);
-        // println!("{:?}", new_enum_table);
         // 第二步，获取这些新枚举到的格子的所有满足周围雷数约束的情况，即子枚举表
         if new_enum_table.is_empty() {
             enum_comb_table.retain(|item| {
@@ -649,7 +688,6 @@ pub fn enum_comb(
                 // 第六步，用本条规则、以及涉及的之前所有规则过滤所有情况
         }
         enum_cell_table.push(enum_cell);
-        println!("enum_comb_table.len = {:?}", enum_comb_table.len());
     }
     enum_comb_table
 }
@@ -777,12 +815,6 @@ fn cal_table_minenum_recursion_step(
             lower_limit = lower_limit_i as usize;
         }
     }
-    // println!("^^^^^lower_limit = {:?}; upper_limit = {:?}", lower_limit, upper_limit);
-    // println!("matrix_b_remain = {:?}", matrix_b_remain);
-    // println!("equation_to_cell_map = {:?}", equation_to_cell_map);
-    // println!("cell_to_equation_map = {:?}", cell_to_equation_map);
-    // println!("mine_vec = {:?}", mine_vec);
-    // println!("idx = {:?}", idx);
 
     for u in lower_limit..upper_limit + 1 {
         // let b = mine_vec[idx];
@@ -809,9 +841,6 @@ fn cal_table_minenum_recursion_step(
             matrix_b_remain[*tt] += u as i32;
         }
         mine_vec[idx] = 0;
-        // println!("回退idx = {:?}", idx);
-        // println!("回退mine_vec = {:?}", mine_vec);
-        // println!("回退matrix_b_remain = {:?}", matrix_b_remain);
     }
     false
 }
@@ -825,10 +854,6 @@ pub fn cal_table_minenum_recursion(
     // 递归算法，得到雷数分布表和每格是雷情况数表，顺便计算最小、最大雷数
     // 输入矩阵必须是非空的，且行列数必须匹配
     // 行数和列数至少为1
-    // println!("combination_relationship = {:?}", combination_relationship);
-    // println!("matrixx_squeeze = {:?}", matrixx_squeeze);
-    // println!("matrix_b = {:?}", matrix_b);
-    // println!("matrixA_squeeze = {:?}", matrixA_squeeze);
     let cells_num = matrixx_squeeze.len();
     if cells_num > 60 {
         // 超出枚举极限长度异常
@@ -862,8 +887,6 @@ pub fn cal_table_minenum_recursion(
         &equation_to_cell_map,
         &mut (vec![0; cells_num]),
     );
-    // println!("{:?}", table_minenum);
-    // println!("{:?}", table_cell_minenum);
     while table_minenum[1][0] == 0 {
         table_minenum[0].remove(0);
         table_minenum[1].remove(0);
@@ -878,8 +901,6 @@ pub fn cal_table_minenum_recursion(
             table_minenum[0].pop();
             table_minenum[1].pop();
             table_cell_minenum.pop();
-            // println!("{:?}", table_minenum);
-            // println!("{:?}", table_cell_minenum);
         }
     }
     if flag_legal_board {
@@ -889,7 +910,6 @@ pub fn cal_table_minenum_recursion(
     }
 }
 
-#[cfg(any(feature = "py", feature = "js", feature = "rs"))]
 pub fn cal_table_minenum_enum(
     matrixA_squeeze: &Vec<Vec<i32>>,
     matrixx_squeeze: &Vec<(usize, usize)>,
@@ -912,11 +932,6 @@ pub fn cal_table_minenum_enum(
         // 无解局面
         return Err(1);
     }
-    // println!("全部情况：{:?}", enum_comb_table);
-    // println!("matrixA_squeeze={:?}", matrixA_squeeze);
-    // println!("matrixx_squeeze={:?}", matrixx_squeeze);
-    // println!("matrix_b={:?}", matrix_b);
-    // println!("combination_relationship={:?}", combination_relationship);
     for s in enum_comb_table.clone() {
         // println!("s: {:?}", s);
         let s_sum = s.iter().sum::<u8>();
@@ -924,13 +939,10 @@ pub fn cal_table_minenum_enum(
         for s_i in 0..s.len() {
             si_num *= C_query(combination_relationship[s_i].len(), s[s_i]);
         }
-        // println!("si_num = {:?}", si_num);
         let fs = table_minenum[0]
             .clone()
             .iter()
             .position(|x| *x == s_sum.into());
-        // println!("table_minenum = {:?}", table_minenum);
-        // println!("fs = {:?}", fs);
         match fs {
             None => {
                 table_minenum[0].push(s_sum.into());
@@ -954,8 +966,6 @@ pub fn cal_table_minenum_enum(
                     }
                 }
                 table_cell_minenum.push(ss);
-                // println!("table_minenum: {:?}", table_minenum);
-                // println!("table_cell_minenum:{:?}", table_cell_minenum);
             }
             _ => {
                 table_minenum[1][fs.unwrap()] += si_num as usize;
@@ -983,10 +993,9 @@ pub fn cal_table_minenum_enum(
     Ok((table_minenum, table_cell_minenum))
 }
 
-/// 用几种模板，检测实际局面中是否有明显的死猜的结构。不考虑起手位置，因为起手必开空。
+/// 用几种模板，检测实际局面中是否有明显的死猜的结构。不考虑起手位置，因为起手必开空。  
 /// 局面至少大于4*4。
-pub fn unsolvableStructure(BoardCheck: &Vec<Vec<i32>>) -> bool {
-    
+pub fn unsolvable_structure(BoardCheck: &Vec<Vec<i32>>) -> bool {
     let row = BoardCheck.len();
     let column = BoardCheck[0].len();
     let mut Board = vec![vec![0; column]; row];
